package com.swagger.rest.controllers

import com.swagger.rest.models.User
import com.swagger.rest.models.UserInput
import com.swagger.rest.repositories.UserRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.crypto.password.PasswordEncoder
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api/v1")
class UserController(private val userRepository: UserRepository) {
    @Autowired
    private val passwordEncoder: PasswordEncoder? = null

    @GetMapping("/users")
    fun getUser(@RequestParam(required = false) username: String?): Any {
        return try {
            val users: List<User> = if (username == null) {
                userRepository.findAll() as List<User>
            } else {
                userRepository.findByUsernameContaining(username) as List<User>
            }
            if (users.isNotEmpty()) {
                ResponseEntity<List<User>?>(users, HttpStatus.OK)
            } else {
                arrayOf<String>()
            }
        } catch (e: Exception) {
            ResponseEntity<List<User>?>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @GetMapping("/users/{id}")
    fun getUserById(@PathVariable("id") id: Long): ResponseEntity<User> {
        val userData: Optional<User?> = userRepository.findById(id)
        return if (userData.isPresent) {
            ResponseEntity<User>(userData.get(), HttpStatus.OK)
        } else {
            ResponseEntity<User>(HttpStatus.NOT_FOUND)
        }
    }

    @PostMapping("/users")
    fun AddUser(@RequestBody user: User): ResponseEntity<User> {
        return try {
            val found = userRepository.findByUsernameContaining(user.username).size
            if (user.username.isNotBlank()) {
                if (user.username.length > 100 || user.password.length > 100 || user.name.length > 255) {
                    ResponseEntity<User>(HttpStatus.PAYLOAD_TOO_LARGE)
                } else if (found > 0) {
                    ResponseEntity<User>(HttpStatus.CONFLICT)
                } else {
                    user.username = user.username.trim()
                    user.password = passwordEncoder!!.encode(user.password)
                    val saveUser = userRepository.save(user)
                    ResponseEntity<User>(saveUser, HttpStatus.CREATED)
                }
            } else {
                ResponseEntity<User>(HttpStatus.BAD_REQUEST)
            }
        } catch (e: Exception) {
            ResponseEntity<User>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PutMapping("/users/{id}")
    fun updateUserById(@PathVariable("id") id: Long, @RequestBody user: User): ResponseEntity<out Any?> {
        val userData = userRepository.findById(id)
        val found = userRepository.findByUsernameContaining(user.username).size
        return if (user.username.isNotBlank()) {
            if (user.username.length > 100 || user.name.length > 255) {
                ResponseEntity<User>(HttpStatus.PAYLOAD_TOO_LARGE)
            } else if (userData.isPresent) {
                if ((user.username == userData.get().username && found > 0) || (user.username !== userData.get().username && found == 0)) {
                    val saveUser = userData.get()
                    saveUser.username = user.username.trim()
                    saveUser.name = user.name
                    ResponseEntity<Any?>(userRepository.save(saveUser), HttpStatus.OK)
                } else {
                    ResponseEntity<User>(HttpStatus.CONFLICT)
                }
            } else {
                ResponseEntity<User>(HttpStatus.NOT_FOUND)
            }
        } else {
            ResponseEntity<User?>(HttpStatus.BAD_REQUEST)
        }
    }

    @DeleteMapping("/users/{id}")
    fun deleteUserById(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val find = userRepository.findById(id)
        return try {
            if (find.isPresent) {
                userRepository.deleteById(id)
                ResponseEntity(HttpStatus.OK)
            } else {
                ResponseEntity(HttpStatus.NOT_FOUND)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PutMapping("/users/{id}/password")
    fun updatePassword(@PathVariable("id") id: Long, @RequestBody userInput: UserInput): ResponseEntity<out Any?> {
        val userData = userRepository.findById(id)
        return if (userInput.newPassword!!.isNotBlank() || userInput.oldPassword!!.isNotBlank()) {
            if (userInput.newPassword!!.length > 100) {
                ResponseEntity<User>(HttpStatus.PAYLOAD_TOO_LARGE)
            } else if (userData.isPresent) {
                if (passwordEncoder!!.matches(userInput.oldPassword, userData.get().password)) {
                    val saveUser = userData.get()
                    saveUser.password = passwordEncoder.encode(userInput.newPassword)
                    ResponseEntity<Any?>(userRepository.save(saveUser), HttpStatus.OK)
                } else {
                    ResponseEntity<User>(HttpStatus.FORBIDDEN)
                }
            } else {
                ResponseEntity<User>(HttpStatus.NOT_FOUND)
            }
        } else {
            ResponseEntity<User>(HttpStatus.BAD_REQUEST)
        }
    }
}